package main;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

import java.io.File;

import common.BinderException;
import common.Report;

import config.Settings;
import processing.ContentProc;
import processing.DocumentProc;
import processing.SingleFileProc;
import processing.FileDirPropertiesProc;
import processing.TreeProc;

public class MarkupBinder
{

	public static void main(String[] args)
	{
		try
		{
			Report.info("Start processing...");
			if (args.length > 2)
			{
				Report.error("Error: Too many arguments");
				terminateArgsError();
			}
			String sourcePath = null;
			boolean version = false;
			boolean all = false;
			boolean web = false;
			boolean clean = false;
			String singleConversion = null;
			String flag;
			for (int n = 0; n < args.length; n++)
			{
				String arg = args[n];
				if (arg.startsWith("-"))
				{
					flag = arg.replaceAll("^-", "");
					if (flag.equals(""))
					{
						Report.error("Error: Illegal, empty flag argument");
						terminateArgsError();
					}
					if ((flag.equals("version")) || (flag.equals("v")))
						version = true;
					else if ((flag.equals("all")) || (flag.equals("a")))
						all = true;
					else if ((flag.equals("web")) || (flag.equals("w")))
						web = true;
					else if ((flag.equals("clean")) || (flag.equals("c")))
						clean = true;
					else if (flag.contains("2"))
						singleConversion = flag;
					else
					// Illegal flag
					{
						Report.error("Error: Illegal flag argument '-" + flag + "'");
						terminateArgsError();
					}
				} else
				// sourcePath
				{
					if (sourcePath == null)
						sourcePath = args[n];
					else
					// Too many sourcePath arguments
					{
						Report.error("Error: Illegal additional source argument " + args[n]);
						terminateArgsError();
					}
				}
			}
			processArguments(sourcePath, version, all, web, clean, singleConversion);
			if (Report.isErrorDetected() == false)
			{
				Thread.sleep(500); // Allow err and out streams to synchronize
				Report.info("Processing completed successfully");
			}
		} catch (Exception ex)
		{
			Report.error(ex.toString());
			terminateProcessingError();
		}
		if (Report.isErrorDetected())
			terminateProcessingError();
		terminateSuccess();
	}

	private static void processArguments(String sourcePath, boolean version, boolean all, boolean web, boolean clean,
			String singleConversion) throws Exception
	{
		if (version)
		{
			Report.info(Settings.VERSION + "\n" + Settings.COPYRIGHT);
			return;
		} else if (sourcePath == null)
			sourcePath = (new File(".")).getAbsolutePath(); // Use the current directory
		System.setProperty("debug.source", "Source: " + sourcePath);
		if (singleConversion != null)
		{
			processSingleConversion(singleConversion, sourcePath);
			return;
		}
		if ((all) && (web))
		{
			Report.error("Error: Choose either the -all or the -web flag but not both");
			return;
		}
		if (all)
		{
			System.setProperty("debug.operation.mode", "Mode: " + "Build Binder");
			buildBinder(sourcePath);
			return;
		}
		if (web)
		{
			System.setProperty("debug.operation.mode", "Mode: " + "Rebuild Web Binder");
			rebuildWebBinder(sourcePath);
			return;
		}
		if (clean)
		{
			System.setProperty("debug.operation.mode", "Mode: " + "Clean Binder");
			cleanBinder(sourcePath);
			return;
		}
		// Determine if operation on file or directory
		File sourceFile = new File(sourcePath);
		if (sourceFile.isFile())
		{
			// Insert file should be processed as a directory
			if (sourcePath.toLowerCase().endsWith(Settings.INSERT_FILE_SUFFIX))
			{
				System.setProperty("debug.operation.mode", "Mode: " + "Build folder including insert file");
				buildDirectory(sourcePath);
				return;
			}
			System.setProperty("debug.operation.mode", "Mode: " + "Build document page");
			buildDocumentPage(sourcePath);
			return;
		}
		System.setProperty("debug.operation.mode", "Mode: " + "Build Folder");
		buildDirectory(sourcePath);
	}

	public static void terminateArgsError()
	{
		Report.usage();
		System.exit(1);
	}

	public static void terminateProcessingError()
	{
		System.exit(2);
	}

	public static void terminateSuccess()
	{
		System.exit(0);
	}

	public static void buildDocumentPage(String sourcePath) throws Exception
	{
		TreeProc treeProc = new TreeProc();
		File sourceFile = new File(sourcePath);
		if (sourceFile.exists() == false)
			throw new BinderException("Error in buildDocumentPage:\n path not foune " + sourcePath);
		if (sourceFile.isAbsolute() == false)
		{
			sourcePath = sourceFile.getAbsolutePath();
			sourceFile = new File(sourcePath);
		}
		String sourceDirectoryPath = sourceFile.getParent();
		treeProc.initializeBinderTree(sourceDirectoryPath);
		Settings.setEditModeProperties();
		/*
		 * The following line retrieves the properties files from the whole binder which is not necessary
		 * This was left as is for simplicity, but a special method can be constructed to load only the relevant properties files. 
		 */
		FileDirPropertiesProc.getInstance().loadFileDirPropertiesAllBinderTree(sourceDirectoryPath);
		DocumentProc documentProc = new DocumentProc();
		documentProc.makeFilePage(sourcePath);
	}

	public static void buildContentPage(String sourcePath) throws Exception
	{
		TreeProc treeProc = new TreeProc();
		File sourceFile = new File(sourcePath);
		if (sourceFile.exists() == false)
			throw new BinderException("Error in buildContentPage:\n path not foune " + sourcePath);
		if (sourceFile.isAbsolute() == false)
		{
			sourcePath = sourceFile.getAbsolutePath();
			sourceFile = new File(sourcePath);
		}
		String sourceDirectoryPath = sourceFile.getParent();
		treeProc.initializeBinderTree(sourceDirectoryPath);
		Settings.setEditModeProperties();
		/*
		 * As commented below this method works also in Web mode if required
		 * String targetBaseHTMLDirectoryPath = fTreeProc.getTargetBaseHTMLDirectoryPath(new File (sourcePath).getParent());  
		 * Settings.setWebModeProperties(targetBaseHTMLDirectoryPath);
		 */
		/*
		 * The following line retrieves the properties files from the whole binder which is not necessary
		 * This was left as is for simplicity, but a special method can be constructed to load only the relevant properties files. 
		 */
		FileDirPropertiesProc.getInstance().loadFileDirPropertiesAllBinderTree(sourceDirectoryPath);
		ContentProc contentProc = new ContentProc();
		contentProc.makeFilePage(sourcePath);
	}

	public static void buildDirectory(String sourcePath) throws Exception
	{
		/*
		 * This build is partial 
		 * from the current directory including sub-directories
		 * This build is incremental useful for editting
		 */
		File sourceFile = new File(sourcePath);
		if (sourceFile.isAbsolute() == false)
		{
			sourcePath = sourceFile.getAbsolutePath();
			sourceFile = new File(sourcePath);
		}
		String sourceDirectoryPath = sourcePath;
		if (sourceFile.isFile())
			sourceDirectoryPath = sourceFile.getParent();
		TreeProc treeProc = new TreeProc();
		treeProc.initializeBinderTree(sourceDirectoryPath);
		Settings.setEditModeProperties();
		FileDirPropertiesProc.getInstance().loadFileDirPropertiesAllBinderTree(sourceDirectoryPath);
		treeProc.processPartialBinderTree(sourceDirectoryPath);
	}

	public static void buildBinder(String sourcePath) throws Exception
	{
		File sourceFile = new File(sourcePath);
		if (sourceFile.isAbsolute() == false)
		{
			sourcePath = sourceFile.getAbsolutePath();
			sourceFile = new File(sourcePath);
		}
		String sourceDirectoryPath = sourcePath;
		if (sourceFile.isFile())
			sourceDirectoryPath = sourceFile.getParent();
		TreeProc treeProc = new TreeProc();
		treeProc.initializeBinderTree(sourceDirectoryPath);
		Settings.setEditModeProperties();
		/* 
		 * Edit mode delete Binder HTML directory 
		 * as it is not up-to-date 
		 */
		treeProc.deleteTargetBinderWebDirectory(sourceDirectoryPath);
		FileDirPropertiesProc.getInstance().loadFileDirPropertiesAllBinderTree(sourceDirectoryPath);
		treeProc.processAllBinderTree(sourceDirectoryPath);
	}

	public static void rebuildWebBinder(String sourcePath) throws Exception
	{
		File sourceFile = new File(sourcePath);
		if (sourceFile.isAbsolute() == false)
		{
			sourcePath = sourceFile.getAbsolutePath();
			sourceFile = new File(sourcePath);
		}
		String sourceDirectoryPath = sourcePath;
		if (sourceFile.isFile())
			sourceDirectoryPath = sourceFile.getParent();
		TreeProc treeProc = new TreeProc();
		treeProc.initializeBinderTree(sourceDirectoryPath);
		String targetBaseHTMLDirectoryPath = treeProc.getTargetBaseWebDirectoryPath(sourceDirectoryPath);
		Settings.setWebModeProperties(targetBaseHTMLDirectoryPath);
		/* 
		 * Web mode delete directory first, 
		 * once deleted this call is ignored 
		 */
		treeProc.deleteTargetBinderWebDirectory(sourceDirectoryPath);
		FileDirPropertiesProc.getInstance().loadFileDirPropertiesAllBinderTree(sourceDirectoryPath);
		treeProc.processAllBinderTree(sourceDirectoryPath);
	}

	public static void cleanBinder(String sourcePath) throws Exception
	{
		/*
		 * Cleans both the binder itself and its web representation if exists
		 */
		File sourceFile = new File(sourcePath);
		if (sourceFile.isAbsolute() == false)
		{
			sourcePath = sourceFile.getAbsolutePath();
			sourceFile = new File(sourcePath);
		}
		String sourceDirectoryPath = sourcePath;
		if (sourceFile.isFile())
			sourceDirectoryPath = sourceFile.getParent();
		TreeProc treeProc = new TreeProc();
		treeProc.initializeBinderTree(sourceDirectoryPath);
		Settings.setEditModeProperties();
		treeProc.deleteTargetBinderWebDirectory(sourceDirectoryPath);
		treeProc.cleanAllBinderTree(sourceDirectoryPath, Settings.FILES_EXCLUDED_FROM_CLEANING);
	}

	public static void processSingleConversion(String command, String sourcePath) throws Exception
	{
		File sourceFile = new File(sourcePath);
		if (sourceFile.exists() == false)
		{
			throw new BinderException("Error in processSingleConversion:\n path not foune " + sourcePath);
		}
		if (sourceFile.isFile() == false)
		{
			throw new BinderException("Error in processSingleConversion:\n  path must point to a file not a folder");
		}
		if (sourceFile.isAbsolute() == false)
			sourcePath = sourceFile.getAbsolutePath();
		SingleFileProc singleFileProc = new SingleFileProc();
		singleFileProc.convertFile(command, sourcePath);
	}

}
